from lxml.etree import parse
from meresco.components.http import utils as httputils
from meresco.core import Observable
from urlparse import urlsplit

CRLF = "\r\n"

class Ore(Observable):
    def __init__(self, infra = None):
        Observable.__init__(self)
        self.infra = infra
        self.serverConfig  = self.infra.configRoot
        self.baseurl = self.serverConfig.get('baseurl', section='system')
        self.nsMap = self.serverConfig.getSectionDict('namespaces')

    def handleRequest(self, RequestURI='', **kwargs):
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        if Path.startswith("/ore/") and Path.endswith(".rdf"):
            recordId = Path.split("/ore/")[1].split(".rdf")[0]
            if not self.any.recordAccessAllowed(recordId, kwargs):
                yield forbiddenXml()
                yield """<?xml version="1.0" encoding="UTF-8"?>
<denied/>"""
                return
            yield self.makeResourceMap(recordId)
        elif Path.startswith('/ore/'):
            yield "HTTP/1.1 303 See Other" + CRLF
            yield "Location: " + self.baseurl + Path + ".rdf" + CRLF
        elif Path.startswith('/pub/'):
            recordId = Path.split("/pub/")[1]
            yield "HTTP/1.1 303 See Other" + CRLF
            yield "Location: " + self.baseurl + "/ore/" + recordId + ".rdf" + CRLF

    def makeResourceMap(self, recordId):
        meta = parse(self.any.getStream(recordId, 'meta'))
        parts = meta.xpath("/meta:meta/meta:parts/meta:part/text()", self.nsMap)
        record = self.any.createRecord(recordId)
        values = {}
        timestamp = parse(self.any.getStream(recordId, 'timestamp'))
        values["modified"] = timestamp.xpath("/timestamp/iso/text()")[0].split()[0]
        metadata = parse(self.any.getStream(recordId, 'metadata'))
        mods = parse(self.any.getStream(recordId, 'mods'))
        values["genre"] = mods.xpath("/mods:mods/mods:genre/text()", self.nsMap)[0]
        values["pid"]   = metadata.xpath("/oai:metadata/didl:DIDL/didl:Item/didl:Descriptor/didl:Statement/dii:Identifier/text()", self.nsMap)[0]
        URIs = {}
        URIs["aggregation"]    = self.baseurl + "/ore/" + recordId
        URIs["rem"]            = URIs["aggregation"] + ".rdf"
        URIs["pub"]            = self.baseurl + "/pub/" + recordId
        URIs["mods"]           = self.baseurl + "/mods/" + recordId + ".xml"
        URIs["humanStartPage"] = self.baseurl + "/search?displayType=single&amp;query=" + recordId
        for part in parts:
            if part == "ddi":
                URIs["ddi"] = self.baseurl + "/ddi/" + recordId + ".xml"
            elif part == "ddi3":
                URIs["ddi3"] = self.baseurl + "/ddi3/" + recordId + ".xml"
        fulltextURIs = []
        if 'URL(s)' in record:
            for url in record['URL(s)']:
                if url.find("arno.uvt.nl") > -1:
                    fulltextURIs.append(url)
        authors = []
        names = mods.xpath("/mods:mods/mods:name", self.nsMap)
        for name in names:
            if name.xpath("mods:role/mods:roleTerm[@type='code']/text()", self.nsMap)[0] == "aut":
                given  = name.xpath("mods:namePart[@type='given']/text()", self.nsMap)[0]
                family = name.xpath("mods:namePart[@type='family']/text()", self.nsMap)[0]
                ID = name.xpath("@ID")[0]
                xpath = "/mods:mods/mods:extension/dai:daiList/dai:identifier[@IDref='%s']" %ID
                DAIs = mods.xpath(xpath, self.nsMap)
                dai_nl = ""
                anr = ""
                for dai in DAIs:
                    if dai.xpath("@authority") and dai.xpath("@authority")[0] == "info:eu-repo/dai/nl":
                        dai_nl = dai.text
                    else:
                        anr = dai.text.split("/dai/")[1]
                authors.append({'given':given, 'family': family, 'anr': anr, 'dai': dai_nl})         

        r = "HTTP/1.0 200 OK" + CRLF + \
            "Content-Type: application/rdf+xml; charset=utf-8" + CRLF + \
            CRLF + \
            '<?xml version="1.0" encoding="UTF-8"?>'
        r += """
<rdf:RDF
 xmlns:dai="http://purl.org/info:eu-repo/dai#"
 xmlns:dcterms="http://purl.org/dc/terms/"
 xmlns:foaf="http://xmlns.com/foaf/0.1/"
 xmlns:ore="http://www.openarchives.org/ore/terms/"
 xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 xmlns:wdrs="http://www.w3.org/2007/05/powder-s#"
 >
 <!-- Resource Map -->
 <rdf:Description rdf:about="%(rem)s">
  <ore:Describes rdf:resource="%(aggregationURI)s"/>
  <dcterms:creator rdf:resource="%(baseurl)s"/>
  <dcterms:modified>%(modified)s</dcterms:modified>
 </rdf:Description>
 %(aggregation)s
 %(publication)s
 <!-- Publisher -->
 <rdf:Description rdf:about="_:publisher">
  <rdf:type rdf:resource="http://xmlns.com/foaf/0.1/Organization"/>
  <foaf:name>Tilburg University. Library and IT Services</foaf:name>
 </rdf:Description>
 %(authors)s
 <!-- MODS -->
 <rdf:Description rdf:about="%(modsURI)s">
  <rdf:type rdf:resource="http://purl.org/info:eu-repo/semantics/descriptiveMetadata"/>
  <ore:isAggregatedBy>%(aggregationURI)s</ore:isAggregatedBy>
  <dcterms:relation>%(pubURI)s</dcterms:relation>
  <dcterms:publisher rdf:resource="_:publisher"/>
  <dcterms:title>MODS metadata of &quot;%(title)s&quot;</dcterms:title>
  <dcterms:format>text/xml</dcterms:format>
  <dcterms:conformsTo>http://www.loc.gov/mods/v3</dcterms:conformsTo>
  <dcterms:modified>%(modified)s</dcterms:modified>
 </rdf:Description>
 %(ddi)s
 %(objectFiles)s
 <!-- Human start page -->
 <rdf:Description rdf:about="%(hSP)s">
  <rdf:type rdf:resource="http://purl.org/info:eu-repo/semantics/humanStartPage"/>
  <dcterms:publisher rdf:resource="_:publisher"/>
 </rdf:Description>
</rdf:RDF>""" %{'aggregation'   : self.aggregation(fulltextURIs, record, URIs, values),
                'aggregationURI': URIs["aggregation"],
                'authors'       : self.authors(authors),
                'baseurl'       : self.baseurl,
                'ddi'           : self.ddi(record, URIs, values),
                'hSP'           : URIs["humanStartPage"],
                'modified'      : values["modified"],
                'modsURI'          : URIs["mods"],
                'objectFiles'   : self.objectFiles(fulltextURIs, record, values),
                'pid'           : values["pid"],
                'publication'   : self.publication(authors, record, URIs, values),
                'pubURI'        : URIs['pub'],
                'rem'           : URIs["rem"],
                'title'         : record["Title"]
                }
        return r

### Aggregation ###
    def aggregation(self, fulltextURIs, record, URIs, values):
        objectFiles = ""
        if len(fulltextURIs) > 0:
            for url in fulltextURIs:
                objectFiles += """
  <ore:aggregates rdf:resource="%s"/>""" %url
        ddi = ""
        for d in ['ddi', 'ddi3']:
            if d in URIs:
                ddi += """
  <ore:aggregates rdf:resource="%s"/>""" %URIs[d]
        return """
 <!-- Aggregation -->
 <rdf:Description rdf:about="%(aggregation)s">
  <rdf:type rdf:resource="http://www.openarchives.org/ore/terms/Aggregation" />
  <ore:isDescribedBy rdf:resource="%(rem)s"/>
  <dcterms:publisher rdf:resource="_:publisher"/>
  <ore:similarTo rdf:resource="%(pid)s"/>
  <dcterms:title>%(title)s</dcterms:title>
  <dcterms:modified>%(modified)s</dcterms:modified>
  <ore:aggregates rdf:resource="%(pub)s"/>
  <ore:aggregates rdf:resource="%(mods)s"/>
  <ore:aggregates rdf:resource="%(hSP)s"/>
  %(objectFiles)s
  %(ddi)s
 </rdf:Description>""" %{
        'aggregation': URIs["aggregation"],
        'ddi'        : ddi,
        'modified'   : values["modified"], 
        'pid'        : values["pid"], 
        'rem'        : URIs["rem"],
        'pub'        : URIs["pub"],
        'mods'       : URIs['mods'],
        'hSP'        : URIs["humanStartPage"],
        'objectFiles': objectFiles,
        'title'      : record["Title"], 
        }

### Publication ###
    def publication(self, authors, record, URIs, values):
        r = """
 <!-- Publication -->
 <rdf:Description rdf:about="%(publication)s">
  <rdf:type rdf:resource="http://purl.org/%(genre)s"/>
  <ore:isAggregatedBy>%(aggregation)s</ore:isAggregatedBy>
  <wdrs:descibedBy>%(mods)s</wdrs:descibedBy>
  <ore:similarTo rdf:resource="%(pid)s"/>
  <dcterms:title>%(title)s</dcterms:title>
  <dcterms:issued>%(issued)s</dcterms:issued>""" %{
            'aggregation':URIs["aggregation"],
            'genre'      :values["genre"],
            'issued'     :record["Publication date"], 
            'mods'       :URIs["mods"],
            'pid'        :values["pid"], 
            'publication':URIs["pub"], 
            'title'      :record["Title"], 
        }
        for author in authors:
            if author["anr"]:
                resource = "http://www.tilburguniversity.edu/nl/webwijs/show/?anr=" + author["anr"]
                r += """
  <dcterms:creator rdf:resource="%s"/>""" %resource
            else:
                r += """
  <dcterms:creator>%s %s</dcterms:creator>""" %(author["given"], author["family"])
        if "Source" in record:
            r += """
  <dcterms:isPartOf>%s</dcterms:isPartOf>""" %record["Source"].replace("<i>", "&quot;").replace("</i>", "&quot;")
        if "Abstract" in record:
            r += """
  <dcterms:description>%s</dcterms:description>""" %record["Abstract"]
        r += """
 </rdf:Description>"""
        return r

### Object Files ###
    def objectFiles(self, fulltextURIs, record, values):
        r = ""
        for url in fulltextURIs:
            r += """
 <rdf:Description rdf:about="%(url)s">
  <rdf:type rdf:resource="http://purl.org/info:eu-repo/semantics/objectFile"/>
  <dcterms:title>Fulltext of &quot;%(title)s&quot;</dcterms:title>
  <dcterms:publisher rdf:resource="_:publisher"/>
  <dcterms:modified>%(modified)s</dcterms:modified>
 </rdf:Description>""" %{
            'modified':values["modified"],
            'title'   :record["Title"],
            'url'     :url
        }
        return r

### Authors ###
    def authors(self, authors):
        r = ""
        for author in authors:
            if author["anr"]:
                about = "http://www.tilburguniversity.edu/nl/webwijs/show/?anr=" + author["anr"]
                dai = ""
                if author["dai"]:
                    dai = "<dai:daiId>info:eu-repo/dai/nl/" + author["dai"] + "</dai:daiId>"
                r += """
 <rdf:Description rdf:about="%(about)s">
  <rdf:type rdf:resource="http://xmlns.com/foaf/0.1/Person"/>
  <foaf:familyName>%(family)s</foaf:familyName>
  <foaf:givenName>%(given)s</foaf:givenName>
  %(dai)s
 </rdf:Description> """ %{
                    'about' : about,
                    'family': author["family"],
                    'given' : author["given"],
                    'dai'   : dai
                }
                return r

### ddi ###
    def ddi(self, record, URIs, values):
        r = ""
        url = ""
        if "ddi" in URIs: url = URIs["ddi"]
        elif "ddi3" in URIs: url = URIs["ddi3"]
        if url:
            r += """
 <rdf:Description rdf:about="%(url)s">
  <rdf:type rdf:resource="http://purl.org/info:eu-repo/semantics/descriptiveMetadata"/>
  <ore:isAggregatedBy>%(aggregationURI)s</ore:isAggregatedBy>
  <dcterms:relation>%(pubURI)s</dcterms:relation>
  <dcterms:publisher rdf:resource="_:publisher"/>
  <dcterms:title>DDI description of data used by &quot;%(title)s&quot;</dcterms:title>
  <dcterms:format>text/xml</dcterms:format>
  <dcterms:conformsTo>http://www.icpsr.umich.edu/DDI</dcterms:conformsTo>
  <dcterms:modified>%(modified)s</dcterms:modified>
 </rdf:Description>""" %{
            'aggregationURI' : URIs["aggregation"],
            'modified'       : values["modified"],
            'pubURI'         : URIs["pub"],
            'title'          : record["Title"],
            'url'            : url
        }
        return r
